import logging
import time
import uuid

from flask import Blueprint, request, g, render_template, send_from_directory

from kunegu import query, storage, entities, c, util, pipeline, debugutil
from kunegu import planets
from kunegusr.ksrutil import no_cache
from kunegusr.app import ksrconfig

log = logging.getLogger('kunegusr.views')


bp_ksr = Blueprint('bp_ksr', 'kunegusr.views', template_folder='templates')


### signals

@c.ksr_app.before_request
def fill_planet():
    planet_name = request.args.get('p')
    if not planet_name:
        if getattr(ksrconfig, 'get_default_planet'):
            g.planet = ksrconfig.get_default_planet()
        else:
            g.planet = None
    else:
        g.planet = planets.fetch_planet(planet_name)
    if not g.planet:
        raise ValueError('No planet configured. Set the ksrconfig.DEFAULT_PLANET_NAME variable.')


### Helpers


class ToDebug(object):
    item_prefix = 'debug-'

    def __init__(self, args):
        self.items = {}
        for arg, value in args.items():
            if arg.startswith(self.item_prefix):
                self.items[arg[len(self.item_prefix):]] = value
        self.kinds = args.get(self.item_prefix + 'entkinds', '').strip().split()

    @staticmethod
    def from_request():
        if ksrconfig.DEBUG_SEARCH_RESULTS:
            return ToDebug(request.args)
        return None

    @property
    def all_planet_names(self):
        return sorted(c.storage.select_planet_names())

    @property
    def domains_of_current_planet(self):
        return c.storage.select_domains(g.planet)



### Static files

@bp_ksr.route('/static/<path:filename>')
def static(filename):
    return send_from_directory('static', filename)

@bp_ksr.route('/favicon.ico')
def favicon():
    return send_from_directory('static', 'img/favicon.ico')


### Main page

@bp_ksr.route('/', methods=['GET', 'POST'])
def search():
    start = time.time()

    query_object = query.Query(g.planet, request.args.get('q', ''))

    try:
        page_no = int(request.args['page_no'])
    except:
        page_no = 0

    if not query_object.is_empty():
        search_results_page = g.planet.index.search(g.planet, query_object, page_no,
                                                    ksrconfig.RESULTS_PER_PAGE,
                                                    ksrconfig.ONE_RESULT_PER_DOMAIN)
    else:
        search_results_page = None

    text = render_template('search.html',
                           query_object=query_object,
                           page_no=page_no,
                           results_per_page=ksrconfig.RESULTS_PER_PAGE,
                           search_results_page=search_results_page,
                           to_debug=ToDebug.from_request())

    elapsed = '%.3f' % (time.time() - start)
    text = text.replace('__kunegu_elapsed', elapsed)
    return text


### Debugging

@no_cache
@bp_ksr.route('/debug/extraction_result/<entity_id_hex>', methods=['GET'])
def debug_blocks(entity_id_hex):
    entity_id = uuid.UUID(entity_id_hex)
    target_ent = util.first(entities.fetch_entities_with_single_id(g.planet, 'processed', entity_id))
    if not target_ent:
        source_ent = util.first(entities.fetch_entities_with_single_id(
            g.planet, 'extracted', entity_id))
        if not source_ent:
            return 'No processed or extracted entity available', 404
        source_ent_pipeline = pipeline.SingleIdPipeline(g.planet, entity_list=[source_ent],
                                                        pipeline_run_tags={'full_pipeline'})
        source_ent_pipeline.run_pipeline(['processed'])
        target_ent = source_ent_pipeline.get('processed')
    return debugutil.generate_blocks_html(target_ent.extraction_result,
        'extraction_result for %s %s %s' % (entity_id.hex, target_ent.kind, target_ent.planet))

